@@ -55,14 +55,12 @@ import java.util.Comparator;
 import java.util.Date;
 import java.util.List;
 import com.openexchange.ajax.container.ByteArrayFileHolder;
-import com.openexchange.ajax.container.IFileHolder;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.contact.ContactService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.contact.helpers.ContactField;
 import com.openexchange.groupware.container.Contact;
-import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.search.ContactSearchObject;
 import com.openexchange.halo.HaloContactDataSource;
 import com.openexchange.halo.HaloContactImageSource;
@@ -73,13 +71,13 @@ import com.openexchange.tools.iterator.SearchIterator;
 import com.openexchange.tools.session.ServerSession;
 
 public class ContactDataSource implements HaloContactDataSource, HaloContactImageSource {
-    
+
     private ServiceLookup services = null;
-    
+
     public ContactDataSource(ServiceLookup services) {
         this.services = services;
     }
-    
+
 	@Override
 	public AJAXRequestResult investigate(HaloContactQuery query, AJAXRequestData req, ServerSession session)
 			throws OXException {
@@ -109,53 +107,73 @@ public class ContactDataSource implements HaloContactDataSource, HaloContactImag
 
     @Override
     public Picture getPicture(HaloContactQuery contactQuery, ServerSession session) throws OXException {
+        return getPicture0(contactQuery, session, true);
+    }
+
+    @Override
+    public String getPictureETag(HaloContactQuery contactQuery, ServerSession session) throws OXException {
+        final Picture picture = getPicture0(contactQuery, session, true);
+        return null == picture ? null : picture.getEtag();
+    }
+
+    private Picture getPicture0(HaloContactQuery contactQuery, ServerSession session, boolean withBytes) throws OXException {
         List<Contact> mergedContacts = contactQuery.getMergedContacts();
         Collections.sort(mergedContacts,  new ImagePrecedence());
-        
+
         for (final Contact contact : mergedContacts) {
             if (contact.getImage1() != null) {
-                ByteArrayFileHolder holder = new ByteArrayFileHolder(contact.getImage1());
-                holder.setContentType(contact.getImageContentType());
-                holder.setName("image");
-                
-                return new Picture(contact.getParentFolderID() + "/" + contact.getObjectID() + "/" + contact.getLastModified().getTime(), holder);
+                final ByteArrayFileHolder holder;
+                if (withBytes) {
+                    holder = new ByteArrayFileHolder(contact.getImage1());
+                    holder.setContentType(contact.getImageContentType());
+                    holder.setName("image");
+                } else {
+                    holder = null;
+                }
+
+                return new Picture(buildETagFor(contact), holder);
             }
         }
-        
+
         // Try with explicit load
         for (Contact c : mergedContacts) {
             final Contact contact = getContact(session, c.getParentFolderID()+"", c.getObjectID()+"");
             if (contact.getImage1() != null) {
-                ByteArrayFileHolder holder = new ByteArrayFileHolder(contact.getImage1());
-                holder.setContentType(contact.getImageContentType());
-                holder.setName("image");
-                
-                return new Picture(contact.getParentFolderID() + "/" + contact.getObjectID() + "/" + contact.getLastModified().getTime(), holder);
+                final ByteArrayFileHolder holder;
+                if (withBytes) {
+                    holder = new ByteArrayFileHolder(contact.getImage1());
+                    holder.setContentType(contact.getImageContentType());
+                    holder.setName("image");
+                } else {
+                    holder = null;
+                }
+
+                return new Picture(buildETagFor(contact), holder);
             }
         }
-        
+
         Contact contact = contactQuery.getContact();
-        
-        Picture p = searchByMailAddress(session, contact.getEmail1());
+
+        Picture p = searchByMailAddress(session, contact.getEmail1(), withBytes);
         if (p != null) {
             return p;
         }
-        
-        p = searchByMailAddress(session, contact.getEmail2());
+
+        p = searchByMailAddress(session, contact.getEmail2(), withBytes);
         if (p != null) {
             return p;
         }
-        
-        p = searchByMailAddress(session, contact.getEmail3());
+
+        p = searchByMailAddress(session, contact.getEmail3(), withBytes);
         if (p != null) {
             return p;
         }
         // Give up
-        
+
         return null;
     }
-    
-    private Picture searchByMailAddress(ServerSession session, String email) throws OXException {
+
+    private Picture searchByMailAddress(ServerSession session, String email, boolean withBytes) throws OXException {
         if (email == null) {
             return null;
         }
@@ -164,44 +182,74 @@ public class ContactDataSource implements HaloContactDataSource, HaloContactImag
         cso.setEmail2(email);
         cso.setEmail3(email);
         cso.setOrSearch(true);
-        
+
         SearchIterator<Contact> result = services.getService(ContactService.class).searchContacts(session, cso, new ContactField[]{ContactField.FOLDER_ID, ContactField.IMAGE1, ContactField.IMAGE1_CONTENT_TYPE, ContactField.LAST_MODIFIED});
         if (result == null) {
             return null;
         }
-        
+
         List<Contact> contacts = new ArrayList<Contact>();
         while(result.hasNext()) {
             Contact contact = result.next();
-            if (contact.getImage1() != null) {
+
+            if (contact.getImage1() != null && (checkEmail(contact, email))) {
                 contacts.add(contact);
             }
         }
         Collections.sort(contacts, new ImagePrecedence());
-        
+
         for (Contact contact : contacts) {
-            ByteArrayFileHolder holder = new ByteArrayFileHolder(contact.getImage1());
-            holder.setContentType(contact.getImageContentType());
-            holder.setName("image");
-            
-            return new Picture(contact.getParentFolderID() + "/" + contact.getObjectID()+ "/" + contact.getLastModified().getTime(), holder);
+            final ByteArrayFileHolder holder;
+            if (withBytes) {
+                holder = new ByteArrayFileHolder(contact.getImage1());
+                holder.setContentType(contact.getImageContentType());
+                holder.setName("image");
+            } else {
+                holder = null;
+            }
+
+            return new Picture(buildETagFor(contact), holder);
         }
-        
+
         return null;
     }
 
+    private boolean checkEmail(Contact c, String email) {
+        if (c.getEmail1() != null && c.getEmail1().equalsIgnoreCase(email)) {
+            return true;
+        }
+        if (c.getEmail2() != null && c.getEmail2().equalsIgnoreCase(email)) {
+            return true;
+        }
+        if (c.getEmail3() != null && c.getEmail3().equalsIgnoreCase(email)) {
+            return true;
+        }
+        return false;
+    }
+
     private Contact getContact(ServerSession session, String folderId, String id) throws OXException {
         return services.getService(ContactService.class).getContact(session, folderId, id);
     }
-    
+
+    private static String buildETagFor(final Contact contact) {
+        return null == contact ? null : new StringBuilder(512).append(contact.getParentFolderID()).append('/').append(contact.getObjectID()).append('/').append(contact.getLastModified().getTime()).toString();
+    }
+
     private static class ImagePrecedence implements Comparator<Contact> {
 
+        /**
+         * Initializes a new {@link ContactDataSource.ImagePrecedence}.
+         */
+        ImagePrecedence() {
+            super();
+        }
+
         @Override
         public int compare(Contact o1, Contact o2) {
             if (o1.getParentFolderID() == 6 && o2.getParentFolderID() != 6) {
                 return -1;
             }
-            
+
             if (o1.getParentFolderID() != 6 && o2.getParentFolderID() == 6) {
                 return 1;
             }
