@@ -53,6 +53,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -66,6 +67,7 @@ import com.openexchange.config.cascade.ConfigView;
 import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.contact.ContactService;
 import com.openexchange.exception.OXException;
+import com.openexchange.groupware.contact.helpers.ContactField;
 import com.openexchange.groupware.contact.helpers.ContactMerger;
 import com.openexchange.groupware.container.Contact;
 import com.openexchange.groupware.ldap.User;
@@ -76,6 +78,7 @@ import com.openexchange.halo.HaloContactImageSource;
 import com.openexchange.halo.HaloContactQuery;
 import com.openexchange.halo.HaloExceptionCodes;
 import com.openexchange.halo.Picture;
+import com.openexchange.java.Strings;
 import com.openexchange.server.ExceptionOnAbsenceServiceLookup;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.iterator.SearchIterator;
@@ -90,12 +93,12 @@ import com.openexchange.user.UserService;
 public class ContactHaloImpl implements ContactHalo {
 
     private final Map<String, HaloContactDataSource> contactDataSources;
-    private final List<HaloContactImageSource> imageSources; 
-    
+    private final List<HaloContactImageSource> imageSources;
+
     private final Lock imageSourcesLock = new ReentrantLock();
-    
+
     private final ServiceLookup services;
-    
+
     /**
      * Initializes a new {@link ContactHaloImpl}.
      *
@@ -120,13 +123,13 @@ public class ContactHaloImpl implements ContactHalo {
         if (!(contact.getInternalUserId() > 0) && !contact.containsEmail1() & !contact.containsEmail2() & !contact.containsEmail3()) {
             throw HaloExceptionCodes.INVALID_CONTACT.create();
         }
-        return dataSource.investigate(buildQuery(contact, session), req, session);
+        return dataSource.investigate(buildQuery(contact, session, true), req, session);
     }
-    
+
     @Override
     public Picture getPicture(Contact contact, ServerSession session) throws OXException {
-        HaloContactQuery contactQuery = buildQuery(contact, session);
-        
+        HaloContactQuery contactQuery = buildQuery(contact, session, true);
+
         for (HaloContactImageSource source : imageSources) {
             if (!source.isAvailable(session)) {
                 continue;
@@ -135,79 +138,115 @@ public class ContactHaloImpl implements ContactHalo {
             if (picture != null){
                 StringBuilder etagBuilder = new StringBuilder();
                 etagBuilder.append(source.getClass().getName()).append("://").append(picture.getEtag());
-                
+
                 picture.setEtag(etagBuilder.toString());
-                
+
                 return picture;
             }
         }
         return null;
     }
-    
+
+    @Override
+    public String getPictureETag(Contact contact, ServerSession session) throws OXException {
+        HaloContactQuery contactQuery = buildQuery(contact, session, false);
+        for (HaloContactImageSource source : imageSources) {
+            if (!source.isAvailable(session)) {
+                continue;
+            }
+            String eTag = source.getPictureETag(contactQuery, session);
+            if (eTag != null) {
+                StringBuilder etagBuilder = new StringBuilder();
+                etagBuilder.append(source.getClass().getName()).append("://").append(eTag);
+                return etagBuilder.toString();
+            }
+        }
+        return null;
+    }
+
     // Friendly for testing
-    HaloContactQuery buildQuery(Contact contact, final ServerSession session) throws OXException {
+    HaloContactQuery buildQuery(final Contact contact, final ServerSession session) throws OXException {
+        return buildQuery(contact, session, true);
+    }
+
+    // Friendly for testing
+    HaloContactQuery buildQuery(final Contact contact, final ServerSession session, final boolean withBytes) throws OXException {
         final UserService userService = services.getService(UserService.class);
         final ContactService contactService = services.getService(ContactService.class);
-
         final HaloContactQuery contactQuery = new HaloContactQuery();
+        final ContactField[] fields = withBytes ? null : new ContactField[] { ContactField.OBJECT_ID, ContactField.LAST_MODIFIED, ContactField.FOLDER_ID };
 
-        if (contact.getObjectID() > 0 && contact.getParentFolderID() > 0) {
-            Contact loaded = contactService.getContact(session, "" + contact.getParentFolderID(), "" + contact.getInternalUserId());
-            contactQuery.setContact(loaded);
-            contactQuery.setMergedContacts(Arrays.asList(loaded));
-            return contactQuery;
+        Contact resultContact = contact;
+
+        // Look-up associated user...
+        User user = null;
+
+        // Prefer look-up by user identifier
+        {
+            final int userId = resultContact.getInternalUserId();
+            if (userId > 0) {
+                user = userService.getUser(userId, session.getContext());
+            }
         }
 
+        // Check by object/folder identifier
+        if (null == user) {
+            if (resultContact.getObjectID() > 0 && resultContact.getParentFolderID() > 0) {
+                Contact loaded = contactService.getContact(session, Integer.toString(resultContact.getParentFolderID()), Integer.toString(resultContact.getObjectID()), fields);
+                contactQuery.setContact(loaded);
+                contactQuery.setMergedContacts(Arrays.asList(loaded));
+                return contactQuery;
+            }
+        }
 
         // Try to find a user with a given eMail address
-
-        User user = null;
-        if (contact.getInternalUserId() > 0) {
-            user = userService.getUser(contact.getInternalUserId(), session.getContext());
-        }
-
-        if (user == null) {
+        if (user == null && resultContact.containsEmail1()) {
             try {
-                user = userService.searchUser(contact.getEmail1(), session.getContext(), false);
+                user = userService.searchUser(resultContact.getEmail1(), session.getContext(), false);
             } catch (final OXException x) {
                 // Don't care. This is all best effort anyway.
             }
         }
 
-        if (user == null) {
+        if (user == null && resultContact.containsEmail2()) {
             try {
-                user = userService.searchUser(contact.getEmail2(), session.getContext(), false);
+                user = userService.searchUser(resultContact.getEmail2(), session.getContext(), false);
             } catch (final OXException x) {
                 // Don't care. This is all best effort anyway.
             }
         }
 
-        if (user == null) {
+        if (user == null && resultContact.containsEmail3()) {
             try {
-                user = userService.searchUser(contact.getEmail3(), session.getContext(), false);
+                user = userService.searchUser(resultContact.getEmail3(), session.getContext(), false);
             } catch (final OXException x) {
                 // Don't care. This is all best effort anyway.
             }
         }
 
         contactQuery.setUser(user);
-        final List<Contact> contactsToMerge = new ArrayList<Contact>();
+        final List<Contact> contactsToMerge = new LinkedList<Contact>();
         if (user != null) {
             // Load the associated contact
-            contact = contactService.getUser(session, user.getId());
-            contactsToMerge.add(contact);
-        } else {
+            resultContact = contactService.getUser(session, user.getId(), fields);
+            contactsToMerge.add(resultContact);
+        } else if (false == Strings.isEmpty(resultContact.getEmail1())){
             // Try to find a contact
             final ContactSearchObject contactSearch = new ContactSearchObject();
-            contactSearch.setEmail1(contact.getEmail1());
-            contactSearch.setEmail2(contact.getEmail1());
-            contactSearch.setEmail3(contact.getEmail1());
+            final String email = resultContact.getEmail1();
+            contactSearch.setEmail1(email);
+            contactSearch.setEmail2(email);
+            contactSearch.setEmail3(email);
             contactSearch.setOrSearch(true);
+            contactSearch.setExactMatch(true);
             SearchIterator<Contact> iterator = null;
             try {
-                iterator = contactService.searchContacts(session, contactSearch);
+                iterator = contactService.searchContacts(session, contactSearch, fields);
                 while (iterator.hasNext()) {
-                    contactsToMerge.add(iterator.next());
+                    Contact c = iterator.next();
+                    if (checkEmails(c, email)) {
+                        contactsToMerge.add(c);
+                    }
                 }
             } finally {
                 if (null != iterator) {
@@ -219,12 +258,27 @@ public class ContactHaloImpl implements ContactHalo {
 
         final ContactMerger contactMerger = new ContactMerger(false);
         for (final Contact c : contactsToMerge) {
-            contact = contactMerger.merge(contact, c);
+            resultContact = contactMerger.merge(resultContact, c);
         }
-        contactQuery.setContact(contact);
+        contactQuery.setContact(resultContact);
         return contactQuery;
     }
 
+    private boolean checkEmails(Contact c, String email1) {
+        if (c.getEmail1() != null && c.getEmail1().equalsIgnoreCase(email1)) {
+            return true;
+        }
+
+        if (c.getEmail2() != null && c.getEmail2().equalsIgnoreCase(email1)) {
+            return true;
+        }
+        if (c.getEmail3() != null && c.getEmail3().equalsIgnoreCase(email1)) {
+            return true;
+        }
+
+        return false;
+    }
+
     @Override
     public List<String> getProviders(final ServerSession session) throws OXException {
         final ConfigViewFactory configViews = services.getService(ConfigViewFactory.class);
@@ -249,7 +303,7 @@ public class ContactHaloImpl implements ContactHalo {
     public void removeContactDataSource(final HaloContactDataSource ds) {
         contactDataSources.remove(ds.getId());
     }
-    
+
     public void addContactImageSource(final HaloContactImageSource is) {
         try {
             imageSourcesLock.lock();
@@ -260,18 +314,18 @@ public class ContactHaloImpl implements ContactHalo {
                 public int compare(HaloContactImageSource o1, HaloContactImageSource o2) {
                     return o2.getPriority() - o1.getPriority();
                 }
-                
+
             });
         } finally {
             imageSourcesLock.unlock();
         }
     }
-    
+
     public void removeContactImageSource(final HaloContactImageSource is) {
         try {
             imageSourcesLock.lock();
             imageSources.remove(is);
-            
+
         } finally {
             imageSourcesLock.unlock();
         }
